@@ -5,15 +5,14 @@ package GPLVote::SignDoc::Client;
 # Public License, Version 3.  Please read the important licensing and
 # disclaimer information included below.
 
-# $Id: Client.pm,v 0.6 2014/12/30 17:23:00 Andrey Velikoredchanin $
+# $Id: Client.pm,v 0.7 2015/02/06 17:23:00 Andrey Velikoredchanin $
 
 use Crypt::OpenSSL::RSA;
 use Crypt::OpenSSL::AES;
 use Crypt::CBC;
 use Bytes::Random::Secure qw(random_bytes);
 use MIME::Base64;
-use LWP::UserAgent;
-use Digest::SHA qw(sha256_base64);
+use Digest::SHA qw(sha256_base64 sha256);
 use JSON;
 use utf8;
 use Encode;
@@ -25,12 +24,12 @@ use vars qw($VERSION);
 use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
 @ISA         = qw(Exporter);
-@EXPORT      = qw(user_sign_is_valid calc_pub_key_id encrypt send_doc get_docs split_base64);
-@EXPORT_OK   = qw(user_sign_is_valid calc_pub_key_id encrypt send_doc get_docs split_base64);
-%EXPORT_TAGS = (DEFAULT => [qw(&user_sign_is_valid &calc_pub_key_id &encrypt &send_doc &get_docs &split_base64)]);
+@EXPORT      = qw(user_sign_is_valid calc_pub_key_id encrypt split_base64);
+@EXPORT_OK   = qw(user_sign_is_valid calc_pub_key_id encrypt split_base64);
+%EXPORT_TAGS = (DEFAULT => [qw(&user_sign_is_valid &calc_pub_key_id &encrypt &split_base64)]);
 
 BEGIN {
-  $VERSION = '0.6';
+  $VERSION = '0.7';
 }
 
 =head1 NAME
@@ -50,18 +49,6 @@ GPLVote::SignDoc::Client -  module for helping create GPLVote SignDoc client sof
  my $pub_key_id = calc_pub_key_id($public_key);
 
  my $enc_data = encrypt($public_key, $data);
-
-
- def get_one_doc {
-    my ($doc) = @_;
-
-    push(@global_array, $doc);
- };
-
- get_doc('123', \&get_one_doc);
- foreach (my $doc (@global_array)) {
-    ... process $doc
- };
  
 =head1 Methods
 
@@ -92,38 +79,6 @@ Encrypt data over public key.
 
 Returning plain Base64 string with encrypted data.
 
-=head2 send_doc(hash document, client_password)
-
-Send document to proxy server.
-
-document - hash with structured document description like:
-
-* type - type of document (for example, "SIGN_REQUEST")
-
-* site - client site identification name (for example, "www.site.com")
-
-* data - encrypted document data like JSON array of strings (for example, encode('["string data1", "string data 2", ...]'))
-
-* template - template for show document data, started from template type line (for example, "LIST\nTitle for data1\nTitle for data 2\n...")
-
-* doc_id - internal client document identificator. Will be present in answered SIGN document.
-
-client_password - client password
-
-Return http response from LWP::UserAgent ($returned_value->is_success mean than 200 HTTP response code)
-
-=head2 get_docs(site, client_password, get_one_doc function, unixtime from_time)
-
-site - client site identification name
-
-client_password - client password
-
-function - reference to function with one argument: hash with structured document.
-
-from_time - unix_time of moment after that get all documents
-
-Return 1 if docs present.
-
 =head1 BUGS
 
 No known bugs, but this does not mean no bugs exist.
@@ -167,13 +122,13 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 
 
 sub user_sign_is_valid {
-  my ($b64_pub_key, $b64_sign, $data) = @_;
+  my ($b64_pub_key, $b64_sign, $data, $use_sha256_hash) = @_;
 
   my $b64_open_key = "-----BEGIN PUBLIC KEY-----\n".split_base64($b64_pub_key)."\n-----END PUBLIC KEY-----";
   my $dec_sign = decode_base64($b64_sign);
     
   my $rsa = Crypt::OpenSSL::RSA->new_public_key($b64_open_key);
-
+  $rsa->use_sha256_hash if $use_sha256_hash;
   return($rsa->verify($data, $dec_sign));
 }
 
@@ -212,16 +167,20 @@ sub encrypt {
   if (length($data) <= 256) {
     return(encode_base64($rsa->encrypt($data), ''));
   } else {
-    # TODO: реализовать блочное шифрование большого файла
-    # Первые 256 байт - зашифрованный 256-битный AES ключ
-    # Дальше - данные зашифрованные AES с помощью этого ключа
+    # First 256 bytes - RSA-encrypted header:
+    #   32 bytes - 256-bits AES key
+    #   16 bytes - IV for AES
+    #   32 bytes - sha256 crc for data
     my $enc_data = '';
 
-    # Случайный ключ AES (256 bit)
+    # Random AES (256 bit)
     my $aes_key = random_bytes(32);
-    # Случайный вектор инициализации IV (
+    # Random IV (
     my $aes_iv = random_bytes(16);
-    $enc_data .= $rsa->encrypt($aes_key.$aes_iv);
+    # CRC for data
+    my $crc = sha256($data);
+    
+    $enc_data .= $rsa->encrypt($aes_key.$aes_iv.$crc);
 
     my $aes_cbc = Crypt::CBC->new( -key => $aes_key,
                                     -literal_key => 1,
@@ -236,81 +195,16 @@ sub encrypt {
   };
 };
 
-sub send_doc {
-  my ($doc, $client_password) = @_;
-
-  my $server = "http://signdoc.gplvote.org/send";
-  my $req = HTTP::Request->new(POST => $server);
-  $req->header('content-type' => 'application/json');
-  $req->header('x-password' => $client_password);
-
-  my $data_request = to_json($doc);
-
-  $req->content(encode('UTF-8', $data_request));
-
-  my $ua = LWP::UserAgent->new;
-
-  my $resp = $ua->request($req);
-
-  if (!$resp->is_success) {
-    warn "HTTP request error ".$resp->code." - ".$resp->message;
-  }
-
-  return($resp);
-};
-
-sub get_docs {
-  my ($site, $client_password, $get_doc_func, $from_time) = @_;
-
-  $from_time = time() - 3600*24 if !defined($from_time);
-  
-  my $doc_request = {
-    type => 'CLIENT_REQUEST',
-    site => $site,
-    from_time => $from_time,
-  };
-  
-  my $server = "http://signdoc.gplvote.org/get";
-  my $req = HTTP::Request->new(POST => $server);
-  $req->header('content-type' => 'application/json');
-  $req->header('x-password' => $client_password);
-  
-  $req->content(to_json($doc_request));
-
-  my $ua = LWP::UserAgent->new;
-
-  my $resp = $ua->request($req);
-
-  my $docs_present;
-  if (!$resp->is_success) {
-    warn "HTTP request error ".$resp->code." - ".$resp->message;
-  } else {
-    my $response = from_json($resp->content);
-  
-    if ($response->{status} eq '0') {
-      # Перебираем документы и в зависимости от типа обрабатываем
-      foreach my $doc (@{$response->{documents}}) {
-        $docs_present = 1;
-        $get_doc_func->($doc) if defined($get_doc_func);
-      };
-    } else {
-      warn "Bad request auth status";
-    };
-  };
-  
-  return($docs_present);
-};
-
 sub to_hash {
   my ($json) = @_;
   my $h;
   my $js = JSON->new();
-  # позволяет обработать невалидный json
+  # for invalid json
   $js->relaxed(1);
-  # преобразование в utf-8
+  # convert to utf-8
   $js->utf8;
   eval {
-    # eval нужен для того что-бы не падало приложение при ошибках обработки json
+    # eval required for no exception if bad json
     $h = $js->decode($json);
   };
   undef($js);
@@ -322,13 +216,11 @@ sub from_hash {
 
   my $s = '';
   my $js = JSON->new();
-  # позволяет обработать невалидный json
+  # for invalid json
   $js->relaxed(1);
-  # преобразование в utf-8
-  # $js->utf8;
   $js->pretty(1) if ($pretty);
   eval {
-    # eval нужен для того что-бы не падало приложение при ошибках обработки json
+    # eval required for no exception if bad json
     $s = $js->encode($h);
   };
   undef($js);
@@ -336,5 +228,4 @@ sub from_hash {
   return($s);
 };
 
-
 1;
\ No newline at end of file
